@@ -1,17 +1,24 @@
 import re
 
-
 class Element(object):
     """contains the pieces of an element and can populate itself from haml element text"""
     
-    self_closing_tags = ('meta', 'img', 'link', 'br', 'hr', 'input')
+    self_closing_tags = ('meta', 'img', 'link', 'br', 'hr', 'input', 'source', 'track')
 
     ELEMENT = '%'
     ID = '#'
     CLASS = '.'
 
-    HAML_REGEX = re.compile(r"(?P<tag>%\w+(\:\w+)?)?(?P<id>#[\w-]*)?(?P<class>\.[\w\.-]*)*(?P<attributes>\{.*\})?(?P<selfclose>/)?(?P<django>=)?(?P<inline>[^\w\.#\{].*)?")
-    
+    HAML_REGEX = re.compile(r"""
+    (?P<tag>%\w+(\:\w+)?)?
+    (?P<id>\#[\w-]*)?
+    (?P<class>\.[\w\.-]*)*
+    (?P<attributes>\{.*\})?
+    (?P<selfclose>/)?
+    (?P<django>=)?
+    (?P<inline>[^\w\.#\{].*)?
+    """, re.X|re.MULTILINE|re.DOTALL)
+
     def __init__(self, haml):
         self.haml = haml
         self.tag = None
@@ -60,9 +67,10 @@ class Element(object):
                 text += '_'+one_id
         return text
 
-    def _parse_attribute_dictionary(self, attribute_dict_string):        
+    def _parse_attribute_dictionary(self, attribute_dict_string):
         attributes_dict = {}
         if (attribute_dict_string):
+            attribute_dict_string = attribute_dict_string.replace('\n', ' ')
             try:
                 #attribute_dict_string = re.sub(r'=(?P<variable>[a-zA-Z_][a-zA-Z0-9_.]+)', "'{{\g<variable>}}'", attribute_dict_string)
                 # converting all allowed attributes to python dictionary style
