@@ -11,13 +11,15 @@ DOCTYPE = '!!!'
 
 HTML_COMMENT = '/'
 CONDITIONAL_COMMENT = '/['
-HAML_COMMENT = '-#'
+HAML_COMMENTS = ['-#', '=#']
 
 VARIABLE = '='
 TAG = '-'
 
+COFFEESCRIPT_FILTERS = [':coffeescript', ':coffee']
 JAVASCRIPT_FILTER = ':javascript'
 CSS_FILTER = ':css'
+STYLUS_FILTER = ':stylus'
 PLAIN_FILTER = ':plain'
 PYTHON_FILTER = ':python'
 CDATA_FILTER = ':cdata'
@@ -47,21 +49,28 @@ def create_node(haml_line):
     if stripped_line[0] == HTML_COMMENT:
         return CommentNode(haml_line)
     
-    if stripped_line.startswith(HAML_COMMENT):
-        return HamlCommentNode(haml_line)
+    for comment_prefix in HAML_COMMENTS:
+        if stripped_line.startswith(comment_prefix):
+            return HamlCommentNode(haml_line)
     
     if stripped_line[0] == VARIABLE:
         return VariableNode(haml_line)
-    
+
     if stripped_line[0] == TAG:
         return TagNode(haml_line)
     
     if stripped_line == JAVASCRIPT_FILTER:
         return JavascriptFilterNode(haml_line)
     
+    if stripped_line in COFFEESCRIPT_FILTERS:
+        return CoffeeScriptFilterNode(haml_line)
+        
     if stripped_line == CSS_FILTER:
         return CssFilterNode(haml_line)
     
+    if stripped_line == STYLUS_FILTER:
+        return StylusFilterNode(haml_line)
+
     if stripped_line == PLAIN_FILTER:
         return PlainFilterNode(haml_line)
         
@@ -89,7 +98,8 @@ class RootNode:
             self.internal_nodes.append(node)
     
     def _should_go_inside_last_node(self, node):
-        return self.internal_nodes and (node.indentation > self.internal_nodes[-1].indentation or (node.indentation == self.internal_nodes[-1].indentation and self.internal_nodes[-1].should_contain(node)))
+        return self.internal_nodes and (node.indentation > self.internal_nodes[-1].indentation
+            or (node.indentation == self.internal_nodes[-1].indentation and self.internal_nodes[-1].should_contain(node)))
     
     def render(self):
         return self.render_internal_nodes()
@@ -111,7 +121,7 @@ class HamlNode(RootNode):
         self.haml = haml.strip()
         self.raw_haml = haml
         self.indentation = (len(haml) - len(haml.lstrip()))
-        self.spaces = ''.join(' ' for i in range(self.indentation))
+        self.spaces = ''.join(haml[0] for i in range(self.indentation))
     
     def render(self):
         return ''.join([self.spaces, self.haml, '\n', self.render_internal_nodes()])
@@ -230,8 +240,13 @@ class TagNode(HamlNode):
                     'filter':'endfilter',
                     'autoescape':'endautoescape',
                     'with':'endwith',
-                    }
-    may_contain = {'if':'else', 
+                    'blocktrans': 'endblocktrans',
+                    'spaceless': 'endspaceless',
+                    'comment': 'endcomment',
+                    'cache': 'endcache',
+                    'localize': 'endlocalize',
+                    'compress': 'endcompress'}
+    may_contain = {'if':['else', 'elif'], 
                    'ifchanged':'else',
                    'ifequal':'else',
                    'ifnotequal':'else',
@@ -254,7 +269,7 @@ class TagNode(HamlNode):
         return output
     
     def should_contain(self, node):
-        return isinstance(node,TagNode) and self.may_contain.get(self.tag_name,'') == node.tag_name
+        return isinstance(node,TagNode) and node.tag_name in self.may_contain.get(self.tag_name,'')
 
 
 class FilterNode(HamlNode):
@@ -292,6 +307,14 @@ class JavascriptFilterNode(FilterNode):
         return output
         
         
+class CoffeeScriptFilterNode(FilterNode):
+    def render(self):
+        output = '<script type=\'text/coffeescript\'>#<![CDATA[\n'
+        output += ''.join([node.raw_haml for node in self.internal_nodes])
+        output += '\n#]]></script>'
+        return output
+
+
 class CssFilterNode(FilterNode):
     def render(self):
         output = '<style type=\'text/css\'>\n/*<![CDATA[*/\n'
@@ -300,6 +323,15 @@ class CssFilterNode(FilterNode):
         return output
 
 
+class StylusFilterNode(FilterNode):
+    def render(self):
+        output = '<style type=\'text/stylus\'>\n/*<![CDATA[*/\n'
+        first_indentation = self.internal_nodes[0].indentation
+        output += '\n'.join([node.raw_haml[first_indentation:] for node in self.internal_nodes])
+        output += '\n/*]]>*/\n</style>\n'
+        return output
+
+
 class CDataFilterNode(FilterNode):
     def render(self):
         output = self.spaces + '<![CDATA[\n'
